# © 2019 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).
from datetime import datetime

from ocb.odoo import api

from odoo import _, fields, models

MAP_DICT = {"loan": "loan.manager.loan", "invest": "", "warranty": ""}


class FinancialContract(models.Model):
    _inherit = "financial.contract"
    deal_tool_id = fields.Many2one(
        comodel_name="financial.deal.tool",
        string="Deal Tool",
        ondelete="restrict",
    )
    deal_id = fields.Many2one(
        comodel_name="financial.deal",
        string="Deal",
        related="deal_tool_id.deal_id",
        store=True,
    )
    deal_tool_model_id = fields.Many2one(
        "ir.model", compute="_compute_deal_tool_model_id", store=True
    )
    deal_tool_model_id_referenced_id = fields.Integer(
        readonly=True, help="Id of the financial tool for this contract"
    )
    # ------------------------------------------------------
    # SQL Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Default methods
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------
    @api.depends("deal_tool_id")
    def _compute_deal_tool_model_id(self):
        if self.deal_tool_id.type:
            self.deal_tool_model_id = self.env["ir.model"].search(
                [("model", "=", MAP_DICT[self.deal_tool_id.type])]
            )

    # ------------------------------------------------------
    # Onchange / Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # CRUD methods (ORM overrides)
    # ------------------------------------------------------

    @api.model
    def create(self, values):
        # FIXME: ORM overload + design does not imply inheritance,
        #  therefore hardcoded creation of each model ? (Warranty, title and loan)
        contract = super().create(values)
        creation_dict = {}

        match self.deal_tool_model_id.model:
            case str(s) if s == MAP_DICT.get("loan"):
                creation_dict = {
                    "description": contract.name,
                    "first_term_date": datetime.now(),
                    "payment_date": datetime.now(),
                    "amount": contract.deal_tool_id.amount,
                    "partner_id": contract.deal_id.partner_id.id,
                }
            case MAP_DICT.get("title"):
                pass
            case MAP_DICT.get("warranty"):
                pass

        # Create financial tool associated
        financial_tool_record = self.env[contract.deal_tool_model_id.model].create(
            creation_dict
        )
        self.deal_tool_model_id_referenced_id = financial_tool_record.id
        return contract

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------
    def btn_open_tool_form(self):
        return {
            "name": _("Financial tool"),
            "type": "ir.actions.act_window",
            "res_model": self.deal_tool_model_id.model,
            "res_id": self.deal_tool_model_id_referenced_id,
            "view_mode": "form",
        }

    # ------------------------------------------------------
    # Business methods
    # ------------------------------------------------------
